@@ -13,7 +13,7 @@
   ********************************************************************/
 #include <startstop_unix_generic.h>
 
-#define VERSION "v7.1 2007-02-26"
+#define VERSION "v7.3 2008-12-05"
 #define PROGRAM_NAME "startstop"
 
 int RunEarthworm ( int argc, char *argv[] )
@@ -51,10 +51,10 @@ int RunEarthworm ( int argc, char *argv[] )
             exit (0);
         } else {
             strcpy ( metaring.ConfigFile, argv[1] );
-            logit_init( argv[1], (short) metaring.MyModId, MAXLINE*3, metaring.LogSwitch );
         }
     }
 
+   logit_init( metaring.ConfigFile, (short) metaring.MyModId, MAXLINE*3, metaring.LogSwitch );
    fprintf( stderr, "startstop: Using config file %s\n", metaring.ConfigFile );
 
 /* Change working directory to environment variable EW_PARAMS value
@@ -485,12 +485,12 @@ void GetConfig()
                 }
                 str = k_str();
                 if ( !str ) break;
-        if ( strlen( str ) > 3 )
-        {
-          logit( "e" , "startstop: MyClassName <%s> too long; exiting!\n",
-              str );
-          exit( -1 );
-        }
+                if ( strlen( str ) > MAX_CLASSNAME_SIZE )
+                {
+                  logit( "e" , "startstop: MyClassName <%s> too long; exiting!\n",
+                      str );
+                  exit( -1 );
+                }
                 strcpy( parent.className, str );
                 state = READ_MYPRIORITY;
                 break;
@@ -620,7 +620,7 @@ void GetConfig()
                    state = READ_AGENT;
                    break;
                 }
-                if ( strlen ( str ) > 3 )
+                if ( strlen ( str ) > MAX_CLASSNAME_SIZE )
                 {
                     logit( "e" , "startstop: Class name <%s> too long; exiting!\n",
                             str );
@@ -746,6 +746,7 @@ void StopEarthworm()
 {
    int    i;
    int    status;
+   int    ret;
    char   tstr[TIMESTR_LEN];
 #ifdef _USE_POSIX_SIGNALS
    struct    sigaction act;
@@ -765,8 +766,15 @@ void StopEarthworm()
    printf( "Earthworm will stop in %d seconds...\n", metaring.KillDelay );
    for ( i = 0; i < metaring.KillDelay; i++ )
    {
-      if ( waitpid( (pid_t)0, &status, WNOHANG ) == -1 && errno == ECHILD )
+#ifdef _LINUX
+      if ( (ret=waitpid( (pid_t)0, &status, WNOHANG |__WALL )) == -1 && errno == ECHILD )
+#else
+      if ( (ret=waitpid( (pid_t)0, &status, WNOHANG )) == -1 && errno == ECHILD )
+#endif
          break;
+/*
+      fprintf(stderr, "StopEarthworm: SLATE DEBUG: waitpid(0, stat, WNOHANG) returns %d errno=%d\n", ret, errno);
+*/
       sleep_ew( 1000 );
    }
 
@@ -899,7 +907,7 @@ void EncodeStatus( char *statusMsg )
       strcat( statusMsg, line );
    }
 
-   sprintf( line,    "        Stopstart Version:      %s\n", metaring.Version );
+   sprintf( line,    "        Startstop Version:      %s\n", metaring.Version );
    strcat( statusMsg, line );
 
 /* Get and print status of the parent process
@@ -997,7 +1005,10 @@ void EncodeStatus( char *statusMsg )
       sprintf( phrase, " %7d", child[i].pid );
       strcat( line, phrase );
 #ifdef _LINUX /* Linux-only __WALL wait-all */
-      status = waitpid( child[i].pid, &dummy, WNOHANG | __WALL );
+      status = waitpid( child[i].pid, &dummy, WNOHANG | __WALL ); 
+/*
+      fprintf(stderr, "SLATE DEBUG: waitpid( WNOHANG|__WALL) for %d returns %d errno=%d \n", child[i].pid, status, errno);
+*/
 #else /*_LINUX */
       status = waitpid( child[i].pid, &dummy, WNOHANG );
 #endif /*_LINUX */
@@ -1571,6 +1582,64 @@ void SetPriority( pid_t pid, char *ClassName, int Priority )
 #endif
 } /* end SetPriority */
 
+#ifdef _LINUX
+  /*********************************************************************
+				_getLinuxEWStatus()
+
+	get the status by sending a request to the main ring and
+	read it back (overcomes some wacky thread permissions on 
+	calling waitpid() in a thread that didn't do the fork calls
+	
+	this echos the status to stdout and can only be called by 
+	interactive
+   *********************************************************************/
+void _getLinuxEWStatus() {
+   int       rc; 
+   char      msg[5120];
+   long      msgsize = 0L;
+   MSG_LOGO  getLogo[1]; /* if you change this '3' here, you need to change the parameter to tport_getmsg as well */
+   MSG_LOGO  msglogo;
+   int  getStatus;
+   char      message[512];/**/
+   MSG_LOGO  logo;/**/
+
+/* Build status request message
+   ****************************/
+   sprintf(message,"%d\n", metaring.MyModId );
+
+/* Set logo values of message
+   **************************/
+   logo.type   = metaring.TypeReqStatus;
+   logo.mod    = metaring.MyModId;
+   logo.instid = metaring.InstId;
+
+/* Send status message to transport ring
+   **************************************/
+   if ( tport_putmsg( &(metaring.Region[0]), &logo, strlen(message), message ) != PUT_OK )
+   {
+    fprintf(stderr, "status: Error sending message to transport region.\n" );
+   }
+   getLogo[0].type = metaring.TypeStatus;
+   getLogo[0].mod    = metaring.MyModId;
+   getLogo[0].instid = metaring.InstId;
+   getStatus=0;
+   while( getStatus == 0) {
+         rc = tport_getmsg( &(metaring.Region[0]), getLogo, 1,
+                        &msglogo, &msgsize, msg, sizeof(msg)-1 );
+         if( rc==GET_NONE || rc==GET_TOOBIG ) {sleep_ew(1000); continue;}
+         if (msglogo.type == metaring.TypeStatus) {
+                msg[msgsize] ='\0';
+                printf("\n%s", msg);
+                getStatus=1;
+         }
+   }
+
+}
+
+#endif /* _LINUX */
+
+
+	
   /*********************************************************************
    *                           Interactive()                           *
    *                                                                   *
@@ -1579,16 +1648,14 @@ void SetPriority( pid_t pid, char *ClassName, int Priority )
 
 void *Interactive( void *dummy) {
    char ewstat[MAX_STATUS_LEN];
-   char      message[512];/**/
-   MSG_LOGO  logo;/**/
    int  i, scan_return, placeholder;
    char line[MAXLINE];
    char param[MAXLINE];
+   char      message[512]; 	/* used for messages sent to ring for commands*/
+   MSG_LOGO  logo;		/* used to describe the logo of those messages*/
 
 #ifdef _LINUX
-
-   EncodeStatus( ewstat );                          /* One free status */
-   printf( "\n%s\n", ewstat );
+   _getLinuxEWStatus();
 #else /* if not LINUX */
    int z;
    int quit = 0;
@@ -1640,28 +1707,11 @@ void *Interactive( void *dummy) {
       if ( strlen( line ) == 1 )
       {
 #ifdef _LINUX
-
-/* Build status request message
-   ****************************/
-   sprintf(message,"%d\n", metaring.MyModId );
-
-/* Set logo values of message
-   **************************/
-   logo.type   = metaring.TypeReqStatus;
-   logo.mod    = metaring.MyModId;
-   logo.instid = metaring.InstId;
-
-/* Send status message to transport ring
-   **************************************/
-   if ( tport_putmsg( &(metaring.Region[0]), &logo, strlen(message), message ) != PUT_OK )
-   {
-    fprintf(stderr, "status: Error sending message to transport region.\n" );
-    return ( 1 );
-   }
-#endif /* if not _LINUX IGD 2006/11/27 moved the condition up to allow EncodeStatus on Linux*/
+   	_getLinuxEWStatus();
+#else
          EncodeStatus( ewstat );
          printf( "\n%s", ewstat );
-/* #endif IGD 2006/11/27 commented out*/
+#endif
       }
 
 
